__version__ ='0.1.0'
__author__ = 'candyabc'
__copyright__ = 'candyabc'
__email__ = 'hfcandyabc@163.com'
__license__ = 'maxwin'


from .sync_basedata import data_manager

from .consul_handle import consul_reader
from .kong import Kong
# __all__=("JttTaskManager","JttTask")